package container

import (
	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"

	corev1 "k8s.io/api/core/v1"
)

// GenConfigMapFunc defines a function which generates ConfigMap.
type GenContainerFunc func(Conf) (corev1.Container, error)

// GenServicePortsFunc defines a function which generates slice of
// ServicePort for Service
type GenContainerPortsFunc func() ([]corev1.ContainerPort, error)

type GenContainerArgsFunc func() ([]string, error)

type GenContainerResourcesFunc func() (corev1.ResourceRequirements, error)

type GenContainerVolumeMountsFunc func() ([]corev1.VolumeMount, error)

type GenContainerLivenessProbeFunc func() (*corev1.Probe, error)

type GenContainerReadinessProbeFunc func() (*corev1.Probe, error)

type GenContainerEnvVarFunc func() ([]corev1.EnvVar, error)

// Conf is used to pass parameters to functions in this package to
// perform operations on Configmap objects.
type Conf struct {
	Owner           interfaces.Object
	Name            string
	Image           string
	ImagePullPolicy corev1.PullPolicy
	GenContainerFunc
	GenContainerPortsFunc
	GenContainerArgsFunc
	GenContainerResourcesFunc
	GenContainerVolumeMountsFunc
	GenContainerLivenessProbeFunc
	GenContainerReadinessProbeFunc
	GenContainerEnvVarFunc
}

type Mutate interface {
	ContainerOwner() interfaces.Object
	ContainerName() string
	ContainerImage() string
	ContainerImagePullPolicy() corev1.PullPolicy
	// Container() (corev1.Container, error)
	ContainerPorts() ([]corev1.ContainerPort, error)
	ContainerArgs() ([]string, error)
	ContainerResources() (corev1.ResourceRequirements, error)
	ContainerVolumeMounts() ([]corev1.VolumeMount, error)
	ContainerLivennessProbe() (*corev1.Probe, error)
	ContainerReadinessProbe() (*corev1.Probe, error)
	ContainerEnvVar() ([]corev1.EnvVar, error)
}

func (c Conf) ContainerOwner() interfaces.Object {
	return c.Owner
}

func (c Conf) ContainerName() string {
	return c.Name
}
func (c Conf) ContainerImage() string {
	return c.Image
}
func (c Conf) ContainerImagePullPolicy() corev1.PullPolicy {
	return c.ImagePullPolicy
}

//func (c Conf) Container() (corev1.Container, error) {
//	var spec corev1.Container
//	if c.GenContainerFunc != nil {
//		return err,
//	}
//	return spec, nil
//}
func (c Conf) ContainerPorts() ([]corev1.ContainerPort, error) {
	var spec []corev1.ContainerPort
	if c.GenContainerPortsFunc != nil {
		return c.GenContainerPortsFunc()
	}
	return spec, nil
}
func (c Conf) ContainerArgs() ([]string, error) {
	var spec []string
	if c.GenContainerArgsFunc != nil {
		return c.GenContainerArgsFunc()
	}
	return spec, nil
}
func (c Conf) ContainerResources() (corev1.ResourceRequirements, error) {
	var spec corev1.ResourceRequirements
	if c.GenContainerResourcesFunc != nil {
		return c.GenContainerResourcesFunc()
	}
	return spec, nil
}
func (c Conf) ContainerVolumeMounts() ([]corev1.VolumeMount, error) {
	var spec []corev1.VolumeMount
	if c.GenContainerVolumeMountsFunc != nil {
		return c.GenContainerVolumeMountsFunc()
	}
	return spec, nil
}
func (c Conf) ContainerLivennessProbe() (*corev1.Probe, error) {
	var spec *corev1.Probe
	if c.GenContainerLivenessProbeFunc != nil {
		return c.GenContainerLivenessProbeFunc()
	}
	return spec, nil
}
func (c Conf) ContainerReadinessProbe() (*corev1.Probe, error) {
	var spec *corev1.Probe
	if c.GenContainerReadinessProbeFunc != nil {
		return c.GenContainerReadinessProbeFunc()
	}
	return spec, nil
}
func (c Conf) ContainerEnvVar() ([]corev1.EnvVar, error) {
	var spec []corev1.EnvVar
	if c.GenContainerEnvVarFunc != nil {
		return c.GenContainerEnvVarFunc()
	}
	return spec, nil
}
