package job

import (
	"fmt"
	"reflect"

	"github.com/ankitrgadiya/operatorlib/pkg/container"
	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"
	"github.com/ankitrgadiya/operatorlib/pkg/meta"
	"sigs.k8s.io/controller-runtime/pkg/client"
	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"

	pod "github.com/ankitrgadiya/operatorlib/pkg/pod"

	"github.com/pkg/errors"
	batchv1 "k8s.io/api/batch/v1"
	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/client-go/tools/record"
)

func (c Conf) Mutate() controllerutil.MutateFn {

	return func() error {

		// in := c.Obj.(*batchv1.Job)
		out := c.Obj.(*batchv1.Job)

		om, err := meta.MutateObjectMeta(c)

		if err != nil {
			return errors.Wrap(err, "failed to generate objectmeta")
		}

		p, err := pod.MutatePod(c)

		if err != nil {
			return errors.Wrap(err, "failed to generate pod template spec")
		}

		out.ObjectMeta = *om
		out.Spec = batchv1.JobSpec{
			//	Replicas:           c.Size,
			//			Parallelism: c.Strategy,
			//			Completions: metav1.SetAsLabelSelector(om.Labels),
			//			ActiveDeadlineSeconds:
			//			BackoffLimit:
			//			TTLSecondsAfterFinished:
			Selector: metav1.SetAsLabelSelector(om.Labels),
			Template: corev1.PodTemplateSpec{
				ObjectMeta: *om,
				Spec:       p.Spec,
			},
			ManualSelector: &c.ManualSelector,
		}

		//		if in.Spec.Selector != out.Spec.Selector {
		//			out.Spec.Selector = in.Spec.Selector
		//		}
		// mergo.Merge(out, in)
		//fmt.Println(in)
		//fmt.Println(out)
		c.Obj = out
		return err
	}
}

// MaybeUpdate implements MaybeUpdateFunc for Secret object. It
// compares the two Secrets being passed and update the first one if
// required.
func MaybeUpdate(original interfaces.Object, new interfaces.Object) (bool, error) {
	o, ok := original.(*batchv1.Job)
	if !ok {
		return false, errors.New("failed to assert the original object")
	}

	n, ok := new.(*batchv1.Job)
	if !ok {
		return false, errors.New("failed to assert the new object")
	}

	result := reflect.DeepEqual(o.Spec.Template, n.Spec.Template)
	if result {
		return false, nil
	}

	o.Spec = n.Spec

	return true, nil
}

func (c Conf) MakeObject() *batchv1.Job {
	obj := &batchv1.Job{
		ObjectMeta: metav1.ObjectMeta{
			Name:      c.ObjectName(),
			Namespace: c.ObjectNamespace(),
		},
	}

	return obj
}

func (c Conf) Containers() []container.Mutate {
	var spec []container.Mutate
	if c.GenContainers != nil {
		return c.GenContainers
	}
	return spec
}

func (c Conf) Volumes() ([]corev1.Volume, error) {
	var spec []corev1.Volume
	if c.GenVolumesFunc != nil {
		return c.GenVolumesFunc()
	}
	return spec, nil
}
func (c Conf) SecurityContext() (*corev1.PodSecurityContext, error) {
	var spec *corev1.PodSecurityContext
	if c.GenSecurityContextFunc != nil {
		return c.GenSecurityContextFunc()
	}
	return spec, nil
}
func (c Conf) RestartPolicy() (corev1.RestartPolicy, error) {
	var spec corev1.RestartPolicy
	if c.GenRestartPolicyFunc != nil {
		return c.GenRestartPolicyFunc()
	}
	return spec, nil
}

func (c Conf) GenLabelsFunc() (map[string]string, error) {
	var labels map[string]string
	if c.GenLabels != nil {
		return c.GenLabels()
	}
	return labels, nil
}
func (c Conf) GenAnnotationsFunc() (map[string]string, error) {
	var annotations map[string]string
	if c.GenAnnotations != nil {
		return c.GenAnnotations()
	}
	return annotations, nil

}
func (c Conf) GenFinalizersFunc() ([]string, error) {
	if c.GenFinalizers != nil {
		return c.GenFinalizers()
	}
	return nil, nil

}

// Object returns the ObjectSyncer subject
func (s Conf) Object() interfaces.Object {
	return s.Obj
}

// Object returns the ObjectSyncer subject
func (s Conf) ObjectName() string {
	return s.Name
}

// Object returns the ObjectSyncer subject
func (s Conf) ObjectNamespace() string {
	return s.Namespace
}

// ObjectOwner returns the ObjectSyncer owner
func (s Conf) ObjectOwner() interfaces.Object {
	return s.Owner
}

// ObjectWithoutSecretData returns the ObjectSyncer subject without secret data
func (s Conf) ObjectWithoutSecretData() runtime.Object {
	return stripSecrets(s.Obj)
}

// PreviousWithoutSecretData returns the ObjectSyncer previous subject without secret data
func (s Conf) PreviousWithoutSecretData() runtime.Object {
	return stripSecrets(s.previousObject)
}

// ObjectType returns the type of the ObjectSyncer subject
func (s Conf) ObjectType() string {
	return fmt.Sprintf("%T", s.Obj)
}

// OwnerType returns the type of the ObjectSyncer owner
func (s Conf) ObjectOwnerType() string {
	return fmt.Sprintf("%T", s.Owner)
}

// OwnerType returns the type of the ObjectSyncer owner
func (s Conf) SetPreviousObject() {
	//	obj := s.Obj.DeepCopyObject().(*appsv1.Deployment)
	//	s.previousObject = obj
	s.previousObject = s.Obj.DeepCopyObject()
	//s.previousObject = s.Object()
	return
}

// OwnerType returns the type of the ObjectSyncer owner
func (s Conf) GetClient() client.Client {
	return s.Reconcile.GetClient()
}

// OwnerType returns the type of the ObjectSyncer owner
func (s Conf) GetScheme() *runtime.Scheme {
	return s.Reconcile.GetScheme()
}

// OwnerType returns the type of the ObjectSyncer owner
func (s Conf) GetRecorder() record.EventRecorder {
	return s.Reconcile.GetRecorder()
}

// stripSecrets returns the object without secretData
func stripSecrets(obj runtime.Object) runtime.Object {
	// if obj is secret, don't print secret data
	s, ok := obj.(*corev1.Secret)
	if ok {
		s.Data = nil
		s.StringData = nil

		return s
	}

	return obj
}
