package meta

import (
	"fmt"

	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"
	"github.com/pkg/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

var (
	errOwnerDeleted = fmt.Errorf("owner is deleted")

	// ErrIgnore when returned the syncer ignores it and returns nil
	ErrIgnore = fmt.Errorf("ignored error")
)

// GenerateObjectMeta function generates ObjectMeta struct as per the
// Conf struct passed.
func MutateObjectMeta(g Mutate) (om *metav1.ObjectMeta, err error) {
	var labels, annotations map[string]string
	var finalizers []string

	labels, err = g.GenLabelsFunc()
	if err != nil {
		return nil, errors.Wrap(err, "failed to generate labels")
	}

	//	if g.AppendLabels {
	//		err = mergo.Merge(&labels, g.Owner.GetLabels())
	//		if err != nil {
	//			return nil, errors.Wrap(err, "failed to append labels from owner object")
	//		}
	//	}

	annotations, err = g.GenAnnotationsFunc()
	if err != nil {
		return nil, errors.Wrap(err, "failed to generate annotations")
	}

	finalizers, err = g.GenFinalizersFunc()
	if err != nil {
		return nil, errors.Wrap(err, "failed to generate finalizers")
	}

	om = &metav1.ObjectMeta{
		Name:      g.ObjectName(),
		Namespace: g.ObjectNamespace(),
		//	Labels:      labels,
		//	Annotations: annotations,
		//	Finalizers:  finalizers,
	}

	// existingMeta, ok := g.GetObject().(metav1.Object)
	// if !ok {
	//	return om, fmt.Errorf("%s is not a metav1.Object", g.ObjectOwnerType())
	//}

	om.SetLabels(labels)
	om.SetFinalizers(finalizers)
	om.SetAnnotations(annotations)

	return om, nil
}

// ObjectType returns the type of the ObjectSyncer subject
func (s *Conf) ObjectType() string {
	return fmt.Sprintf("%T", s.Object)
}

// OwnerType returns the type of the ObjectSyncer owner
func (s *Conf) ObjectOwnerType() string {
	return fmt.Sprintf("%T", s.Object)
}

// Given an ObjectSyncer, returns a controllerutil.MutateFn which also sets the
// owner reference if the subject has one
//func Mutate(s Mutate) controllerutil.MutateFn {
//	return func() error {
//		obj := s.Object()

//		mutate := s.Mutate()
//	err := mutate()
//	if err != nil {
//		return err
//	}

//		if s.ObjectOwner() != nil {
//			existingMeta, ok := obj.(metav1.Object)
//			if !ok {
//				return fmt.Errorf("%s is not a metav1.Object", s.ObjectType())
//			}

//			ownerMeta, ok := s.ObjectOwner().(metav1.Object)
//			if !ok {
//				return fmt.Errorf("%s is not a metav1.Object", s.ObjectOwnerType())
//			}

// set owner reference only if owner resource is not being deleted, otherwise the owner
// reference will be reset in case of deleting with cascade=false.
//			if ownerMeta.GetDeletionTimestamp().IsZero() {
//				err := controllerutil.SetControllerReference(ownerMeta, existingMeta, s.GetScheme())
//				if err != nil {
//					return err
//				}
//			} else if ctime := existingMeta.GetCreationTimestamp(); ctime.IsZero() {
// the owner is deleted, don't recreate the resource if does not exist, because gc
// will not delete it again because has no owner reference set
//				return errOwnerDeleted
//			}
//		}

//		return nil
//	}
//}

// NewObjectSyncer creates a new kubernetes.Object syncer for a given object
// with an owner and persists data using controller-runtime's CreateOrUpdate.
// The name is used for logging and event emitting purposes and should be an
// valid go identifier in upper camel case. (eg. MysqlStatefulSet)
func NewMetaMutate(name, namespace string, owner interfaces.Object, ownerRef bool, genLabels GenLabels) Mutate {
	return &Conf{
		Owner:          owner,
		Namespace:      namespace,
		Name:           name,
		OwnerReference: ownerRef,
		GenLabels:      genLabels,
	}
}

func (c Conf) ObjectOwner() interfaces.Object {
	return c.Owner
}
func (c Conf) Object() interfaces.Object {
	return c.Obj
}

func (c Conf) ObjectName() string {
	return c.Name
}
func (c Conf) ObjectNamespace() string {
	return c.Namespace
}

func (c Conf) GenLabelsFunc() (map[string]string, error) {
	var labels map[string]string
	if c.GenLabels != nil {
		return c.GenLabels()
	}
	return labels, nil
}
func (c Conf) GenAnnotationsFunc() (map[string]string, error) {
	var annotations map[string]string
	if c.GenAnnotations != nil {
		return c.GenAnnotations()
	}
	return annotations, nil

}
func (c Conf) GenFinalizersFunc() ([]string, error) {
	if c.GenFinalizers != nil {
		return c.GenFinalizers()
	}
	return nil, nil

}
